// Sketch_Picture Effect
//
// Copyright Outerspace Software, all rights reserved

string description = "This effect animates a sketch map as generated by Bixelangelo. The 1st texture must be a sketch map and is used for timing. The 2nd texture is used for the alpha channel, the antialiasing and the colour map. If the FX Colour Map Factor property is set to 0, the 2nd texture can be the same as the 1st texture. The antialiasing quality is best when using a 2nd texture, even when it's the same as the 1st.";

float4x4	WorldViewProjection;
float4		DiffuseCol;
float4		TextureResolution0;
float4		TextureResolution1;

float 		ParticleAge=0; 

bool optimization=true;

float4 Prop0
<
string UIName="Writer";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,0,0);
int UISliders=2;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop1
<
string UIName="Smooth";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0.05,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0.05,0,0,0);

float4 Prop2
<
string UIName="Colour Tip";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,1,1);
int UISliders=3;
float UIScale=255;
bool UIInteger=true;
int UIWidget=2;
> = float4(1,1,1,1);

float4 Prop3
<
string UIName="Colour Map Factor";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

float4 Prop4
<
string UIName="Black Border";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

float4 Prop5
<
string UIName="Particle Grow (grow, static, shrink)";
float UIMin=0;
float UIMax=10;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

float4 Prop6
<
string UIName="Particle Fade (fade-in, static, fade-out)";
float UIMin=0;
float UIMax=10;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = NONE;
   MagFilter = NONE;
   MipFilter = NONE;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture MyTexture1;
sampler MyTextureSampler1 = sampler_state
{
   Texture = <MyTexture1>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float2 tex:		TEXCOORD0;
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;

	output.position = mul(input.position, WorldViewProjection);
	output.tex 	= input.tex;

	return output;
}

float2 GetStage(){
	float2 Stage=Prop0.xy;
	
	//grow
	if(Prop5.x > 0){
		if(ParticleAge < Prop5.x){
			float t = ParticleAge / Prop5.x;
			if(t>1) t=1;
			if(t<0) t=0;
			Stage.x=t;
		}
	}
	//shrink
	if(Prop5.z > 0){
		if(ParticleAge > (Prop5.x-Prop5.y)){
			float t = 1 - (ParticleAge-Prop5.x-Prop5.y) / Prop5.z;
			if(t>1) t=1;
			if(t<0) t=0;
			Stage.y=1-t;
		}
	}

	return Stage;
}

float4 GetColour(float2 inUV, float2 inStage){

	float4 q0=float4(1,1,1,1);
	float4 q1=float4(1,1,1,1);

	if(TextureResolution0.x>0){
		q0=tex2D(MyTextureSampler0,inUV);
		q1=q0;
	}
	if(TextureResolution1.x>0){
		q1 = tex2D(MyTextureSampler1,inUV);
	}

	float p=(q0.x*65536+q0.y*256+q0.z)/65793.0;


	//colour gradient
	float4 c = DiffuseCol*(1-p) + Prop2*p;

	//colour map factor	
	if(TextureResolution1.x>0){
		c=c*(q1*Prop3.x+1-Prop3.x);
	}


	//Border?
	if(Prop4.x>0 && q1.w>=0 && q1.w<1){
		float f=q1.w*q1.w;
//		float f=q1.w;
		c.xyz = c.xyz*f + (c.xyz*(1-Prop4.x))*(1-f);
	}

	if(q1.w<=0){
		c.w=0;
		return c;
	}

	float Smooth=Prop1.x;

	float t=p;

	float r=1;

	// einde
	float b=-Smooth+inStage.x*(1+2*Smooth);
	if(t>=(b+Smooth)) r=0;
	if(t>b && t<b+Smooth) r=1-(t-b)/Smooth;
	
	// begin
	b=-Smooth+inStage.y*(1+2*Smooth);
	if(t<=b) r=0;
	if(t>b && t<b+Smooth) r*=((t-b)/Smooth);

	r*=q1.w;
	c.w=r;
	return c;
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;

	float4 c=float4(0,0,0,1);

	if(TextureResolution0.x>0){
		float2 Stage=GetStage();
		c=GetColour(input.tex, Stage);
	}

	//fade-in
	if(Prop6.x > 0){
		if(ParticleAge < Prop6.x){
			float t = ParticleAge / Prop6.x;
			if(t>1) t=1;
			if(t<0) t=0;
			c.w*=t;
		}
	}
	//fade-out
	if(Prop6.z > 0){
		if(ParticleAge > (Prop6.x-Prop6.y)){
			float t = 1 - (ParticleAge-Prop6.x-Prop6.y) / Prop6.z;
			if(t>1) t=1;
			if(t<0) t=0;
			c.w*=t;
		}
	}

	c.w*=DiffuseCol.w;

	output.color=c;
	return output;
}

technique Shader
{
	pass P0
	{
		ZWriteEnable = FALSE;
		VertexShader = compile vs_3_0 VS();
		PixelShader  = compile ps_3_0 PS();
	}
}
